@@ -302,6 +302,16 @@ static void init_syslog(const plugin_conf_t *conf)
 				facility = LOG_LOCAL6;
 			else if (strcasecmp(conf->args[i], "LOG_LOCAL7") == 0)
 				facility = LOG_LOCAL7;
+			else if (strcasecmp(conf->args[i], "LOG_AUTH") == 0)
+				facility = LOG_AUTH;
+			else if (strcasecmp(conf->args[i], "LOG_AUTHPRIV") == 0)
+				facility = LOG_AUTHPRIV;
+			else if (strcasecmp(conf->args[i], "LOG_DAEMON") == 0)
+				facility = LOG_DAEMON;
+			else if (strcasecmp(conf->args[i], "LOG_SYSLOG") == 0)
+				facility = LOG_SYSLOG;
+			else if (strcasecmp(conf->args[i], "LOG_USER") == 0)
+				facility = LOG_USER;
 			else {
 				syslog(LOG_ERR, 
 					"Unknown log priority/facility %s",
